package xml2fol.DataStructures;

/**
 * A DataStructure that stores for each file a list of passage offsets and lengths
 * @author Alan Woodley
 */

import xml2fol.*;
import xml2fol.DataStructures.OLStruct;
import java.io.*;
import  java.util.*;

public class FileStruct implements Comparable<Object>{
    
    /** The list of passage offset and lengths  */
    private Vector olList;

    /** The highest *file* rank for this particular file across all submissions */
    private int highestFileRank;

    /** The highest *element* rank for this particular file across all submissions */
    private int highestElementRank;

    /** The file id */
    private int fileID;
    
    /**
     * Produces a new FilStruct with a given fileId and element/file ranks
     * Usually only classed when for the first time that the file has been found in the pool
     * @param fileID - The ID of the File
     * @param elementRank - The element rank for the first occurance of the file in pool
     * @param fileRank k - The file rank for the first occurance of the file in pool
     */
    public FileStruct(int fileID, int elementRank, int fileRank){
        olList = new Vector();
        this.fileID = fileID;
        this.highestElementRank = elementRank;
        this.highestFileRank = fileRank;

    }
    
    /**
     * Adds a given passage's given offset and length to the list of occurances and
     * if nessisary updates the file and element rank (if the given file or element rank 
     * is smaller than the current values)
     * @param offset - The passage's offset
     * @param length - The passage's length
     * @param elementRank  - The passage's elementRank
     * @param fileRank  - The passage's fileRank
     */
    public void Add(int offset, int length, int elementRank, int fileRank){
//        System.out.println("Adding " + elementRank + " " + fileRank);
        olList.add(new OLStruct(offset, length));
        if(elementRank < highestElementRank) highestElementRank = elementRank;
        if(fileRank < highestFileRank) highestFileRank = fileRank;        
    }
    
    /**
     * Returns the file's file id
     * @return - fileId
     */
    public int getFileID(){
        return this.fileID;
    }
    
    /**
     * Returns the file's highest *file* rank
     * @return - highestFileRank
     */
    public int getHighestRank(){
        return this.highestFileRank;
    }
    
    /**
     * Returns the file's highest *element* rank
     * @return - highestElementRank
     */
    public int getHighestElementRank(){
        return this.highestElementRank;        
    }
    
    /**
     * Returns the file's highest *file* rank
     * @return - highestFileRank
     */
    public int getHighestFileRank(){
        return this.highestFileRank;        
    }

    /**
     * Returns the list of passage occurances
     * @return - list of passage occurances
     */
    public Vector getOLList(){
        return olList;
    }    
    
    /**
     * Sorts the list of passage occurances
     */
    public void sortOLList(){
        Collections.sort(olList);
    }

    /**
     * Sets the file id
     * @param fileId - the file id to set
     */    
    public void setFileID(int fileID){
        this.fileID = fileID;
    }
    
    /**
     * Sets the ol list
     * @param olList - the olList to set
     */    
    public void setOLList(Vector olList){
        this.olList = olList;
    }    
        


   /**
    * A function that compares the value of two integers
    * @param i - The first integer
    * @param j - The second integer
    * @return - -1 if i is bigger, 0 if they are equal, 1 if i is smaller    
    */                
    public int compareInt(int i, int j){
        final int BEFORE = -1;
        final int EQUAL = 0;
        final int AFTER = 1;
        
        if (i == j ) return EQUAL;
        if (i > j) return AFTER;
        if (i < j) return BEFORE;
        
        return EQUAL; // wont get to
    }


   /**
    * A function to compare this FileStruct with another FileStruct
    * @param otherObj - The other FileStruct
    * @return - -1 if otherObj is BEFORE this; 
    *            0 if they are EQUAL; 
    *            1 if otherObj is AFTER this     
    */            
    public int compareTo(Object otherObj) {
        FileStruct other = (FileStruct) otherObj;
        final int BEFORE = -1;
        final int EQUAL = 0;
        final int AFTER = 1;
           
            
        if ( this == other ) return EQUAL;
        
        int rankCompare = compareInt(this.getHighestRank(), other.getHighestRank());
        if (rankCompare == EQUAL){
              return compareInt(this.fileID, other.fileID);
           } else {
             return rankCompare;
        }                
     }
        
        
    /**
    * A function that compares this FileStruct with another FileStruct. 
    * @param otherObject - The other FileStruct.
    * @return - true if (this FileStruct equals other FileStruct) otherwise false.
    */
    public boolean equals(Object otherObject) {
        if (this == otherObject) return true;
        if ( !(otherObject instanceof FileStruct) ) return false;

        FileStruct other = (FileStruct) otherObject;
        // return false if any members don't match
        if (this.fileID != other.fileID) return false;            
        if (this.highestElementRank != other.highestElementRank) return false;            
        if (this.highestFileRank != other.highestFileRank) return false;            
        if (this.olList != other.olList) return false;            
        
        return true;
    }

        
    /**
    * A class that overrides equals must also override hashCode.
    * @return - a hashcode
    */
    public int hashCode() {
         int result = HashCodeUtil.SEED;
         result = HashCodeUtil.hash(result, this);
         return result;
     }

    
    

}
